Index: src/uk/me/parabola/mkgmap/osmstyle/housenumber/HousenumberGenerator.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/housenumber/HousenumberGenerator.java	(revision 4517)
+++ src/uk/me/parabola/mkgmap/osmstyle/housenumber/HousenumberGenerator.java	(working copy)
@@ -275,7 +275,7 @@
 		if("false".equals(n.getTag(numbersTagKey)))
 			return;
 		
-		if ("true".equals(n.getTag(POIGeneratorHook.AREA2POI_TAG))){
+		if ("true".equals(n.getTag(POIGeneratorHook.TK_M_AREA2POI))){
 			// ignore POI created for buildings
 			return; 		
 		}
Index: src/uk/me/parabola/mkgmap/reader/osm/POIGeneratorHook.java
===================================================================
--- src/uk/me/parabola/mkgmap/reader/osm/POIGeneratorHook.java	(revision 4517)
+++ src/uk/me/parabola/mkgmap/reader/osm/POIGeneratorHook.java	(working copy)
@@ -15,6 +15,7 @@
 
 import java.util.AbstractMap;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.IdentityHashMap;
@@ -26,6 +27,7 @@
 
 import uk.me.parabola.imgfmt.app.Coord;
 import uk.me.parabola.log.Logger;
+import uk.me.parabola.mkgmap.CommandArgs;
 import uk.me.parabola.mkgmap.osmstyle.NameFinder;
 import uk.me.parabola.mkgmap.osmstyle.function.AreaSizeFunction;
 import uk.me.parabola.util.EnhancedProperties;
@@ -52,7 +54,7 @@
  * <br/>
  * <code>add-pois-to-lines</code><br/>
  * Adds POIs to lines. Each POI is tagged with mkgmap:line2poi=true.<br/>
- * The following POIs are created for each line:
+ * The following POIs maybe created for each line:
  * <ul>
  * <li>mkgmap:line2poitype=start: The first point of the line</li>
  * <li>mkgmap:line2poitype=end: The last point of the line</li>
@@ -80,6 +82,8 @@
 	private boolean poisToLinesEnd = false; 
 	private boolean poisToLinesMid = false; 
 	private boolean poisToLinesOther = false; 
+	private boolean poisToLinesTagged = false;
+
 	private NameFinder nameFinder;
 	private AreaSizeFunction areaSizeFunction = new AreaSizeFunction();
 
@@ -86,19 +90,24 @@
 	private Set<String> usedTagsPOI;
 	
 	/** Name of the bool tag that is set to true if a POI is created from an area */
-	public static final short AREA2POI_TAG = TagDict.getInstance().xlate("mkgmap:area2poi");
-	public static final short LINE2POI_TAG = TagDict.getInstance().xlate("mkgmap:line2poi");
-	public static final short LINE2POI_TYPE_TAG  = TagDict.getInstance().xlate("mkgmap:line2poitype");
-	public static final short WAY_LENGTH_TAG  = TagDict.getInstance().xlate("mkgmap:way-length");
+	public static final short TK_M_AREA2POI = TagDict.getInstance().xlate("mkgmap:area2poi");
+	private static final short TK_M_LINE2POI = TagDict.getInstance().xlate("mkgmap:line2poi");
+	private static final short TK_M_LINE2POI_TYPE = TagDict.getInstance().xlate("mkgmap:line2poitype");
+	private static final short TK_M_WAY_LENGTH = TagDict.getInstance().xlate("mkgmap:way-length");
+	private static final short TK_M_COPIED_AS_POI = TagDict.getInstance().xlate("mkgmap:copied-as-poi");
 	
 	@Override
 	public boolean init(ElementSaver saver, EnhancedProperties props, Style style) {
+		final String lineOpt =  "add-pois-to-lines";
 		poisToAreas = props.containsKey("add-pois-to-areas");
-		poisToLines = props.containsKey("add-pois-to-lines");
+		poisToLines = props.containsKey(lineOpt); 
 		if (poisToLines) {
-			String[] opts = {"all"};
-			if (!props.getProperty("add-pois-to-lines").isEmpty()) {
-				opts = props.getProperty("add-pois-to-lines").split(",");
+			// option did not have parameters in older versions
+			List<String> opts;
+			if (!props.getProperty(lineOpt).isEmpty()) {
+				opts = CommandArgs.stringToList(props.getProperty(lineOpt), lineOpt);
+			} else {
+				opts = Arrays.asList("all");
 			}
 			
 			for (String opt : opts) {
@@ -115,18 +124,21 @@
 				case "other":
 					poisToLinesOther= true;
 					break;
+				case "tagged":
+					poisToLinesOther= true;
+					break;
 				case "all":
 					poisToLinesStart= true;
 					poisToLinesEnd= true;
 					poisToLinesMid = true;
 					poisToLinesOther= true;
+					poisToLinesTagged = true;
 					break;
 
 				default:
-					throw new IllegalArgumentException("Invalied argument '"+opt+"' for add-pois-to-lines");
+					throw new IllegalArgumentException("Invalied argument '" + opt + "' for add-pois-to-lines");
 				}
 			}
-			
 		}
 		
 		if (!(poisToAreas || poisToLines)) {
@@ -336,38 +348,44 @@
 		}
 		// add tag mkgmap:cache_area_size to the original polygon so that it is copied to the POI
 		areaSizeFunction.value(polygon);
-		addPOI(polygon, poiCoord, AREA2POI_TAG, 0); 
+		addPOI(polygon, poiCoord, TK_M_AREA2POI, null, true); 
 	}
 	
 	
 	private int addPOItoLine(Way line) {
-		// calculate the middle of the line
+		// calculate the segment lengths and the full way length
 		Coord prevC = null;
-		double sumDist = 0.0;
-		ArrayList<Double> dists = new ArrayList<>(line.getPoints().size()-1);
+		double wayLen = 0.0;
+		ArrayList<Double> dists = new ArrayList<>(line.getPoints().size() - 1);
 		for (Coord c : line.getPoints()) {
 			if (prevC != null) {
 				double dist = prevC.distance(c);
 				dists.add(dist);
-				sumDist+=dist;
+				wayLen += dist;
 			}
 			prevC = c;
 		}
+		String wayLength = String.valueOf(Math.round(wayLen));
 		
 		int countPOIs = 0;
-		if (poisToLinesStart) {
-			Node startNode = addPOI(line, line.getFirstPoint(), LINE2POI_TAG, sumDist);
-			startNode.addTag(LINE2POI_TYPE_TAG, "start");
-			countPOIs++;
+		Node poiNode = null;
+		if (poisToLinesStart || poisToLinesTagged) {
+			poiNode = addPOI(line, line.getFirstPoint(), TK_M_LINE2POI, wayLength, poisToLinesStart);
+			if (poiNode != null) {
+				poiNode.addTag(TK_M_LINE2POI_TYPE, "start");
+				countPOIs++;
+			}
 		}
 
-		if (poisToLinesEnd) {
-			Node endNode = addPOI(line, line.getLastPoint(), LINE2POI_TAG, sumDist);
-			endNode.addTag(LINE2POI_TYPE_TAG, "end");
-			countPOIs++;
+		if (poisToLinesEnd || poisToLinesTagged) {
+			poiNode = addPOI(line, line.getLastPoint(), TK_M_LINE2POI, wayLength, poisToLinesEnd);
+			if (poiNode != null) {
+				poiNode.addTag(TK_M_LINE2POI_TYPE, "end");
+				countPOIs++;
+			}
 		}
 		
-		if (poisToLinesOther && line.getPoints().size() > 2) {
+		if ((poisToLinesOther || poisToLinesTagged) && line.getPoints().size() > 2) {
 			Coord lastPoint = line.getFirstPoint();
 			for (Coord inPoint : line.getPoints().subList(1, line.getPoints().size() - 1)) {
 				if (inPoint.equals(lastPoint)) {
@@ -374,14 +392,16 @@
 					continue;
 				}
 				lastPoint = inPoint;
-				Node innerNode = addPOI(line, inPoint, LINE2POI_TAG, sumDist);
-				innerNode.addTag(LINE2POI_TYPE_TAG, "inner");
-				countPOIs++;
+				poiNode = addPOI(line, inPoint, TK_M_LINE2POI, wayLength, poisToLinesOther);
+				if (poiNode != null) {
+					poiNode.addTag(TK_M_LINE2POI_TYPE, "inner");
+					countPOIs++;
+				}
 			}
 		}
 		if (poisToLinesMid) {
 			Coord midPoint = null;
-			double remMidDist = sumDist / 2;
+			double remMidDist = wayLen / 2;
 			for (int midPos = 0; midPos < dists.size(); midPos++) {
 				double nextDist = dists.get(midPos);
 				if (remMidDist <= nextDist) {
@@ -393,9 +413,11 @@
 			}
 
 			if (midPoint != null) {
-				Node midNode = addPOI(line, midPoint, LINE2POI_TAG, sumDist);
-				midNode.addTag(LINE2POI_TYPE_TAG, "mid");
-				countPOIs++;
+				poiNode = addPOI(line, midPoint, TK_M_LINE2POI, wayLength, poisToLinesMid);
+				if (poiNode != null) {
+					poiNode.addTag(TK_M_LINE2POI_TYPE, "mid");
+					countPOIs++;
+				}
 			}
 		}
 		return countPOIs;
@@ -405,22 +427,35 @@
 	/** Prefix that is added to tags which are copied from the original node. */
 	public static final String FROM_NODE_PREFIX = "mkgmap:from-node:";
 
-	private Node addPOI(Element source, Coord poiCoord, short poiTypeTagKey, double wayLength) {
+	/**
+	 * Create a new node with tags from source and further tags. If a node is created it is added to the saver.
+	 * @param source the source element
+	 * @param poiCoord the location of the new node
+	 * @param poiTypeTagKey tag key that identifies the source
+	 * @param wayLength length of way if POI was generated for a line
+	 * @param force if {@code false}, only create a new of the original location belongs to a tagged node  
+	 * @return new node or null (see force)
+	 */
+	private Node addPOI(Element source, Coord poiCoord, short poiTypeTagKey, String wayLength, boolean force) {
+		Node node = null;
+		if (poiCoord instanceof CoordPOI) {
+			node = ((CoordPOI) poiCoord).getNode();
+		} else {
+			node = coordToNodeMap.get(poiCoord);
+		}
+		
+		if (!force && node == null)
+			return null;
+		
 		Node poi = new Node(source.getOriginalId(), poiCoord);
 		poi.setFakeId();
 		poi.copyTags(source);
 		poi.deleteTag(MultiPolygonRelation.STYLE_FILTER_TAG);
 		poi.addTag(poiTypeTagKey, "true");
-		if (poiTypeTagKey == LINE2POI_TAG) {
-			poi.addTag(WAY_LENGTH_TAG, String.valueOf(Math.round(wayLength)));
+		if (poiTypeTagKey == TK_M_LINE2POI) {
+			poi.addTag(TK_M_WAY_LENGTH, wayLength);
 		}
 		
-		Node node = null;
-		if (poiCoord instanceof CoordPOI) {
-			node = ((CoordPOI) poiCoord).getNode();
-		} else {
-			node = coordToNodeMap.get(poiCoord);
-		}
 		if (node != null) {
 			// add the original tags of the node with the prefix mkgmap:from-node:
 			for (Entry<String, String> entry : node.getTagEntryIterator()) {
@@ -428,6 +463,7 @@
 					poi.addTag(FROM_NODE_PREFIX + entry.getKey(), entry.getValue());
 				}
 			}
+			node.addTag(TK_M_COPIED_AS_POI, "true");
 		} 
 
 		if (log.isDebugEnabled()) {
@@ -489,7 +525,7 @@
 			}
 
 			if (point != null) {
-				Node poi = addPOI(r, point, AREA2POI_TAG, 0);
+				Node poi = addPOI(r, point, TK_M_AREA2POI, null, true);
 				// remove the type tag which makes only sense for relations
 				poi.deleteTag("type");
 				mps2POI++;
